package com.jaywagnon.webservices.configuration;


import org.apache.commons.configuration.EnvironmentConfiguration;
import org.apache.commons.configuration.MapConfiguration;


/**
 * Enumerates the environment variables that are used to configure
 * the application's behavior.
 * <p/>
 * When the .value() method is called, the environment will be queried
 * for a variable with the same name.  If the variable is found, its value
 * is returned.  If it is not found, the default is returned.  The default
 * may be null.
 */
public enum ConfigurationEnum
{
  DATA_SOURCE_URL(null),
  DATA_SOURCE_DRIVER("org.postgresql.Driver"),
  DATA_SOURCE_USER(null),
  DATA_SOURCE_PASSWORD(null),

  HIBERNATE_DIALECT("org.hibernate.dialect.PostgreSQLDialect"),
  HIBERNATE_HDM2DDL_AUTO("validate"),
  HIBERNATE_SHOW_SQL("false");


  private static MapConfiguration configuration;

  private String defaultValue;


  ConfigurationEnum(String defaultValue)
  {
    this.defaultValue = defaultValue;
  }


  /**
   * Returns the default value for this enum entry.  May be null.
   *
   * @return
   */
  public String defaultValue()
  {
    return defaultValue;
  }


  /**
   * Queries the environment for a variable with the same name as the
   * enum entry.  If the variable is found, its value is returned.
   * If it is not found, the default is returned.  The value or the
   * default may be null.
   *
   * @return
   */
  public String value()
  {
    String value = getConfiguration().getString(this.toString());
    return value != null ? value : defaultValue;
  }


  /**
   * Get the configuration object used to look up the variable values.
   * <p/>
   * If not explicitly set (or after explicitly set to null),
   * an EnvironmentConfiguration object is instantiated.
   *
   * @return MapConfiguration
   */
  public static MapConfiguration getConfiguration()
  {
    if(configuration == null)
      configuration = new EnvironmentConfiguration();

    return configuration;
  }


  /**
   * Set the configuration object used to look up the variable values.
   * Set to null to cause a new EnvironmentConfiguration object to
   * be instantiated on the next getConfiguration() or value() call.
   *
   * @param configuration
   */
  public static void setConfiguration(MapConfiguration configuration)
  {
    ConfigurationEnum.configuration = configuration;
  }
}
