package com.jaywagnon.webservices.configuration;


import org.springframework.web.WebApplicationInitializer;
import org.springframework.web.context.ContextLoaderListener;
import org.springframework.web.context.support.AnnotationConfigWebApplicationContext;
import org.springframework.web.servlet.DispatcherServlet;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRegistration;


/**
 * Web Application Initializer class for no-XML servlet context loading.
 */
public class WebAppInitializer
        implements WebApplicationInitializer
{
  /**
   * Executed as part of the servlet context loading.
   * <p/>
   * Creates two contexts: the Application context and the Dispatcher context.
   *
   * @param servletContext
   * @throws ServletException
   */
  @Override
  public void onStartup(ServletContext servletContext)
          throws ServletException
  {
    initializeWebAppContext(servletContext);
    initializeDispatcherContext(servletContext);
  }


  /**
   * Initializes the Application context using the WebAppConfiguration class and
   * registers a loader listened for it with the servlet context.
   *
   * @param servletContext
   */
  private void initializeWebAppContext(final ServletContext servletContext)
  {
    AnnotationConfigWebApplicationContext context = createWebApplicationContext(WebAppConfiguration.class);
    servletContext.addListener(new ContextLoaderListener(context));
  }


  /**
   * Initializes the Dispatcher context using the DispatcherConfiguration class and
   * registers it with the servlet context.  Also binds the dispatcher to the root mapping.
   *
   * @param servletContext
   */
  private void initializeDispatcherContext(final ServletContext servletContext)
  {
    AnnotationConfigWebApplicationContext context = createWebApplicationContext(DispatcherConfiguration.class);
    ServletRegistration.Dynamic dispatcher = servletContext.addServlet("dispatcher", new DispatcherServlet(context));
    dispatcher.setLoadOnStartup(1);
    dispatcher.addMapping("/");
  }


  /**
   * Creates a new annotation-based context using the given @Configuration annotated class.
   *
   * @param configurationClass
   * @return
   */
  public static AnnotationConfigWebApplicationContext createWebApplicationContext(final Class configurationClass)
  {
    AnnotationConfigWebApplicationContext context = new AnnotationConfigWebApplicationContext();
    context.register(configurationClass);
    return context;
  }
}
