package com.jaywagnon.webservices.web.controller;


import com.jaywagnon.webservices.configuration.TestConfiguration;
import com.jaywagnon.webservices.persistence.entity.User;
import com.jaywagnon.webservices.service.ModelNotFoundException;
import com.jaywagnon.webservices.service.UserService;
import com.jaywagnon.webservices.web.model.UserDTO;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.Link;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

import static org.hamcrest.Matchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;


@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = {TestConfiguration.class})
@WebAppConfiguration
public class UsersControllerTest
{
  private static String URL_BASE = "http://localhost/users";


  @Autowired
  private WebApplicationContext context;


  private MockMvc mockMvc;


  @Autowired
  private UserService service;


  @InjectMocks
  private UsersController controller;


  private static User model1 = new User()
          .setId(1L).setCreated(new Date()).setUpdated(new Date()).setFirstName("First").setLastName("Last");
  private static User model2 = new User()
          .setId(2L).setCreated(new Date()).setUpdated(new Date()).setFirstName("Second").setLastName("Last");
  private static List<User> modelList = Arrays.asList(model1, model2);


  @Before
  public void setup()
  {
    Mockito.reset(service);
    mockMvc = MockMvcBuilders.webAppContextSetup(context).build();
  }


  @Test
  public void test_get_All()
          throws Exception
  {
    when(service.findAll()).thenReturn(modelList);
    ResultActions result = mockMvc.perform(get("/users"));
    result.andExpect(status().isOk());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$", hasSize(modelList.size())));
    result.andExpect(jsonPath("$[0].links", hasSize(1)));
    result.andExpect(jsonPath("$[0].links[0].rel", is(Link.REL_SELF)));
    result.andExpect(jsonPath("$[0].links[0].href", is(URL_BASE + "/" + model1.getId())));
    result.andExpect(jsonPath("$[0].created", is(model1.getCreated().getTime())));
    result.andExpect(jsonPath("$[0].updated", is(model1.getUpdated().getTime())));
    result.andExpect(jsonPath("$[0].firstName", is(model1.getFirstName())));
    result.andExpect(jsonPath("$[0].lastName", is(model1.getLastName())));
    result.andExpect(jsonPath("$[1].links", hasSize(1)));
    result.andExpect(jsonPath("$[1].links[0].href", is(URL_BASE + "/" + model2.getId())));
    result.andExpect(jsonPath("$[1].created", is(model2.getCreated().getTime())));
    result.andExpect(jsonPath("$[1].updated", is(model2.getUpdated().getTime())));
    result.andExpect(jsonPath("$[1].firstName", is(model2.getFirstName())));
    result.andExpect(jsonPath("$[1].lastName", is(model2.getLastName())));

    verify(service, times(1)).findAll();
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_get()
          throws Exception
  {
    when(service.find(model1.getId())).thenReturn(model1);
    ResultActions result = mockMvc.perform(get("/users/{id}", model1.getId()));
    result.andExpect(status().isOk());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.links", hasSize(1)));
    result.andExpect(jsonPath("$.links[0].rel", is(Link.REL_SELF)));
    result.andExpect(jsonPath("$.links[0].href", is(URL_BASE + "/" + model1.getId())));
    result.andExpect(jsonPath("$.created", is(model1.getCreated().getTime())));
    result.andExpect(jsonPath("$.updated", is(model1.getUpdated().getTime())));
    result.andExpect(jsonPath("$.firstName", is(model1.getFirstName())));
    result.andExpect(jsonPath("$.lastName", is(model1.getLastName())));

    verify(service, times(1)).find(model1.getId());
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_get_not_found()
          throws Exception
  {
    when(service.find(model1.getId())).thenThrow(new ModelNotFoundException());
    ResultActions result = mockMvc.perform(get("/users/{id}", model1.getId()));
    result.andExpect(status().isNotFound());

    verify(service, times(1)).find(model1.getId());
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_create()
          throws Exception
  {
    User user = new User()
            .setId(1L)
            .setFirstName(model1.getFirstName())
            .setLastName(model1.getLastName());
    user.prePresist();
    user.preUpdate();

    UserDTO formObject = new UserDTO()
            .setFirstName(user.getFirstName())
            .setLastName(user.getLastName());

    when(service.create(formObject)).thenReturn(user);

    ResultActions result = mockMvc.perform(
            post("/users")
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isCreated());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.links", hasSize(1)));
    result.andExpect(jsonPath("$.links[0].rel", is(Link.REL_SELF)));
    result.andExpect(jsonPath("$.links[0].href", is(URL_BASE + "/" + user.getId())));
    result.andExpect(jsonPath("$.created", is(user.getCreated().getTime())));
    result.andExpect(jsonPath("$.updated", is(user.getUpdated().getTime())));
    result.andExpect(jsonPath("$.firstName", is(user.getFirstName())));
    result.andExpect(jsonPath("$.lastName", is(user.getLastName())));

    verify(service, times(1)).create(formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_create_invalid_nulls()
          throws Exception
  {
    UserDTO formObject = new UserDTO()
            .setFirstName(null)
            .setLastName(null);
    ResultActions result = mockMvc.perform(
            post("/users")
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isBadRequest());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.errors", hasSize(2)));
    result.andExpect(jsonPath("$.errors[*].field", containsInAnyOrder("firstName", "lastName")));
    result.andExpect(jsonPath("$.errors[*].message", not(isEmptyOrNullString())));

    verify(service, never()).create(formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_create_invalid_lastName_too_long()
          throws Exception
  {
    UserDTO formObject = new UserDTO()
            .setFirstName(ControllerTestUtil.createString(64 + 1))
            .setLastName(ControllerTestUtil.createString(64 + 1));
    ResultActions result = mockMvc.perform(
            post("/users")
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isBadRequest());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.errors", hasSize(2)));
    result.andExpect(jsonPath("$.errors[*].field", containsInAnyOrder("firstName", "lastName")));
    result.andExpect(jsonPath("$.errors[*].message", not(isEmptyOrNullString())));

    verify(service, never()).create(formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_create_invalid_lastName_blank()
          throws Exception
  {
    UserDTO formObject = new UserDTO()
            .setFirstName("")
            .setLastName("");
    ResultActions result = mockMvc.perform(
            post("/users")
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isBadRequest());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.errors", hasSize(2)));
    result.andExpect(jsonPath("$.errors[*].field", containsInAnyOrder("firstName", "lastName")));
    result.andExpect(jsonPath("$.errors[*].message", not(isEmptyOrNullString())));

    verify(service, never()).create(formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_update()
          throws Exception
  {
    User user = new User()
            .setId(1L)
            .setUpdated(model1.getUpdated())
            .setFirstName(model1.getFirstName())
            .setLastName(model1.getLastName());
    user.prePresist();

    UserDTO formObject = new UserDTO()
            .setFirstName(user.getFirstName())
            .setLastName(user.getLastName());

    when(service.update(user.getId(), formObject)).thenReturn(user);

    ResultActions result = mockMvc.perform(
            put("/users/{id}", user.getId())
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isOk());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.links", hasSize(1)));
    result.andExpect(jsonPath("$.links[0].rel", is(Link.REL_SELF)));
    result.andExpect(jsonPath("$.links[0].href", is(URL_BASE + "/" + user.getId())));
    result.andExpect(jsonPath("$.created", is(user.getCreated().getTime())));
    result.andExpect(jsonPath("$.updated", is(user.getUpdated().getTime())));
    result.andExpect(jsonPath("$.firstName", is(user.getFirstName())));
    result.andExpect(jsonPath("$.lastName", is(user.getLastName())));

    verify(service, times(1)).update(user.getId(), formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_update_not_found()
          throws Exception
  {
    Long id = 1L;
    UserDTO formObject = new UserDTO()
            .setFirstName(model1.getFirstName())
            .setLastName(model1.getLastName());

    when(service.update(id, formObject)).thenThrow(new ModelNotFoundException());

    ResultActions result = mockMvc.perform(
            put("/users/{id}", id)
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isNotFound());

    verify(service, times(1)).update(id, formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_update_invalid_nulls()
          throws Exception
  {
    Long id = 1L;
    UserDTO formObject = new UserDTO()
            .setFirstName(null)
            .setLastName(null);
    ResultActions result = mockMvc.perform(
            put("/users/{id}", id)
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isBadRequest());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.errors", hasSize(2)));
    result.andExpect(jsonPath("$.errors[*].field", containsInAnyOrder("firstName", "lastName")));
    result.andExpect(jsonPath("$.errors[*].message", not(isEmptyOrNullString())));

    verify(service, never()).update(id, formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_update_invalid_too_long()
          throws Exception
  {
    Long id = 1L;
    UserDTO formObject = new UserDTO()
            .setFirstName(ControllerTestUtil.createString(64 + 1))
            .setLastName(ControllerTestUtil.createString(64 + 1));
    ResultActions result = mockMvc.perform(
            put("/users/{id}", id)
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isBadRequest());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.errors", hasSize(2)));
    result.andExpect(jsonPath("$.errors[*].field", containsInAnyOrder("firstName", "lastName")));
    result.andExpect(jsonPath("$.errors[*].message", not(isEmptyOrNullString())));

    verify(service, never()).update(id, formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_update_invalid_blank()
          throws Exception
  {
    Long id = 1L;
    UserDTO formObject = new UserDTO()
            .setFirstName("")
            .setLastName("");
    ResultActions result = mockMvc.perform(
            put("/users/{id}", id)
                    .contentType(UsersController.APPLICATION_JSON_UTF8)
                    .content(ControllerTestUtil.formObjectToJson(formObject)));
    result.andExpect(status().isBadRequest());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.errors", hasSize(2)));
    result.andExpect(jsonPath("$.errors[*].field", containsInAnyOrder("firstName", "lastName")));
    result.andExpect(jsonPath("$.errors[*].message", not(isEmptyOrNullString())));

    verify(service, never()).update(id, formObject);
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_delete()
          throws Exception
  {
    when(service.delete(model1.getId())).thenReturn(model1);
    ResultActions result = mockMvc.perform(delete("/users/{id}", model1.getId()));
    result.andExpect(status().isOk());
    result.andExpect(content().contentType(UsersController.APPLICATION_JSON_UTF8));
    result.andExpect(jsonPath("$.id", is(model1.getId().intValue())));
    result.andExpect(jsonPath("$.created", is(model1.getCreated().getTime())));
    result.andExpect(jsonPath("$.updated", is(model1.getUpdated().getTime())));
    result.andExpect(jsonPath("$.firstName", is(model1.getFirstName())));
    result.andExpect(jsonPath("$.lastName", is(model1.getLastName())));

    verify(service, times(1)).delete(model1.getId());
    verifyNoMoreInteractions(service);
  }


  @Test
  public void test_delete_not_found()
          throws Exception
  {
    when(service.delete(model1.getId())).thenThrow(new ModelNotFoundException());
    ResultActions result = mockMvc.perform(delete("/users/{id}", model1.getId()));
    result.andExpect(status().isNotFound());

    verify(service, times(1)).delete(model1.getId());
    verifyNoMoreInteractions(service);
  }
}
